use crate::commands::{Command, RequestType};
use crate::context::Context;
use crate::Result;
use geoffrey_db::helper::{find_location_by_name_type, load_location};
use geoffrey_models::models::item::ItemListing;
use geoffrey_models::models::locations::{Location, LocationDataDb, LocationType};
use geoffrey_models::models::parameters::restock_params::RestockParameters;
use geoffrey_models::models::player::Player;
use geoffrey_models::models::response::api_error::GeoffreyAPIError;
use geoffrey_models::models::CommandLevel;
use std::collections::HashSet;
use std::sync::Arc;

pub struct Restock {}

impl Command for Restock {
    type Req = RestockParameters;
    type Resp = Location;

    fn command_name() -> String {
        "restock".to_string()
    }

    fn request_type() -> RequestType {
        RequestType::POST
    }

    fn command_level() -> CommandLevel {
        CommandLevel::REGISTERED
    }

    fn run_command(ctx: Arc<Context>, req: &Self::Req, user: Option<Player>) -> Result<Self::Resp> {
        let user = user.unwrap();

        let mut shop = find_location_by_name_type(
            &ctx.db,
            &req.shop_name,
            user.id.unwrap(),
            LocationType::Shop,
        )?;

        if let LocationDataDb::Shop(shop_data) = &mut shop.loc_data {
            let filter = regex::Regex::new(&req.item_name)
                .map_err(|_| GeoffreyAPIError::ParameterInvalid("item_name".to_string()))?;

            let updated_items: HashSet<ItemListing> = shop_data
                .item_listings
                .iter()
                .map(|item| {
                    let mut item = item.clone();
                    if filter.is_match(&item.item.name) {
                        item.restock();
                    }

                    item
                })
                .collect();

            shop_data.item_listings = updated_items;
            let shop = ctx.db.insert(shop)?;

            load_location(&ctx.db, &shop).map_err(|err| err.into())
        } else {
            Err(GeoffreyAPIError::EntryNotFound)
        }
    }
}
